#ifndef SSLCONTEXT_H
#define SSLCONTEXT_H
/// $Id: sslcontext.h,v 1.8 2003/07/23 16:33:05 dvermeir Exp $

/*! \mainpage
\author dvermeir@vub.ac.be

<a href="../download/">Download</a>

\section install_dvssl Installation

Standard. See the INSTALL file in the top directory.
\section Dependencies Dependencies

<a href="http://tinf2.vub.ac.be/~dvermeir/software/dv/dvnet/html/index.html">dvnet</a>,
<a href="http://tinf2.vub.ac.be/~dvermeir/software/dv/dvutil/html/index.html">dvutil</a>,
<a href="http://www.openssl.org/">openssl</a>.

\section intro_dvssl Introduction

This package provides a rudimentary interface to openssl.

<ul>
<li> The classes Dv::Ssl::SslContextV2, Dv::Ssl::SslContextV3 and
     Dv::Ssl::SslContextV23 represenst SSL contexts. They are derived
     from a common base class Dv::Ssl::SslContext.
<li> The class Dv::Ssl::SslSocket represents a client SSL connection. It
     is derived from Dv::Ssl::Socket (from the dvnet package) and thus
     from iostream.
<li> The class Dv::Ssl::SslServerSocket is the SSL equivalent of
     Dv::Net::ServerSocket, from which it is derived.
<li> The class Dv::Ssl::X509Certificate represents a (server) certificate.
</ul>

*/

/*! \file
   This file defines Dv::Ssl::SslContext and its derived classes
   Dv::Ssl::SslContextV2, Dv::Ssl::SslContextV3 and Dv::Ssl::SslContextV23.
*/

#include <string>
#include <stdexcept>

//! Namespace for all dvxyz packages.
namespace Dv {
//! Dv::Ssl is the namespace for this package.
namespace Ssl {
//! Common base class represensting an SSL context.
/*! The constructors for a a Dv::Ssl::SslSocket and a Dv::Ssl::SslServerSocket
    both have a required Dv::Ssl::SslSocket& argument.

    Example usage:
    <ul>
    <li> For a server, the key and certificate file names are obligatory.
    \code
    try {
      SslContextV23	context("key.pem","cert.pem");
      SslServerSocket	server(context,9999);
      ..
      }
    catch (exception& e) {
      ..
      }
    \endcode
    <li> For a client, the key and certificate file are optional (and
        not used if present).
    \code
    try {
      SslContextV23	context;
      SslSocket		client(context,"host.domain",9999);
      ..
      }
    catch (exception& e) {
      ..
      }
    \endcode
    </ul>
    
*/
class SslContext {
public:
  //! Destructor.
  virtual ~SslContext();
  //! Return pointer to en openssl SSL_CTX structure.
  /*! The pointer is declared void to avoid inclusion of openssl
      header files.
  */
  void*	context() { return context_; }

  //! Return name of private key filename or 0 if none.
  const std::string* rsa_private_key_file() const { return rsa_private_key_file_; }
  //! Return name of certificate filename or 0 if none.
  const std::string* certificate_file() const { return certificate_file_; }

protected:
  //! Constructor is protected because only derived objects make sense.
  SslContext() throw (std::runtime_error);
  //! Associated private key file name with this context.
  /*! Fails is filename==0 or SSL_CTX_useBLBLA fails. */
  bool rsa_private_key_file(const char* filename);
  //! Associated certificate file name with this context.
  /*! Fails is filename==0 or SSL_CTX_useBLBLA fails. */
  bool certificate_file(const char* filename);
  //! Pointer to SSL_CTX structure.
  void*	context_;
  /*! The pointer is declared void to avoid inclusion of openssl
      header files.
  */

private:
  SslContext(const SslContext&);	// forbidden
  SslContext&	operator=(const SslContext&); // forbidden

  std::string*	rsa_private_key_file_;
  std::string*	certificate_file_;
};

//! SSl V2 context, see Dv::Ssl::SslContext.
class SslContextV2: public SslContext {
public:
  //! Constructor.
  /*! If the context is to be used with a Dv::Ssl::SslServerSocket object,
      both filename arguments are obligatory. If the context is
      to be used with a Dv::Ssl::SslSocket object, the filename arguments
      are optional (but will not be used).
  */
  SslContextV2(const char* keyfilename=0, const char* certfilename=0)
  							throw (std::runtime_error);
  //! Destructor.
  ~SslContextV2();
};

//! SSl V23 context, see Dv::Ssl::SslContext.
class SslContextV23: public SslContext {
public:
  //! Constructor.
  /*! If the context is to be used with a Dv::Ssl::SslServerSocket object,
      both filename arguments are obligatory. If the context is
      to be used with a Dv::Ssl::SslSocket object, the filename arguments
      are optional (but will not be used).
  */
  SslContextV23(const char* keyfilename = 0, const char* certfilename = 0)
  							throw (std::runtime_error);
  //! Destructor.
  ~SslContextV23();
};

//! SSl V3 context, see Dv::Ssl::SslContext.
class SslContextV3: public SslContext {
public:
  //! Constructor.
  /*! If the context is to be used with a Dv::Ssl::SslServerSocket object,
      both filename arguments are obligatory. If the context is
      to be used with a Dv::Ssl::SslSocket object, the filename arguments
      are optional (but will not be used).
  */
  SslContextV3(const char* keyfilename = 0, const char* certfilename = 0)
  							throw (std::runtime_error);
  //! Destructor.
  ~SslContextV3();
};

}}

#endif

